package com.kxs.common.core.exception;

import com.kxs.common.core.util.R;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.util.Assert;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.security.access.AccessDeniedException;

import java.util.List;

/**
 * <p>
 * 全局异常处理器
 * </p>
 *
 * @author 没秃顶的码农
 * @date 2023/11/04
 */
@Slf4j
@RestControllerAdvice
@Order(10000)
public class GlobalBizExceptionHandler {

	/**
	 * 全局异常.
	 * @param e the e
	 * @return R
	 */
	@ExceptionHandler(Exception.class)
	@ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
	public R handleGlobalException(Exception e) {
		log.error("全局异常信息 ex={}", e.getMessage(), e);

		return R.failed(e.getLocalizedMessage());
	}

	/**
	 * 运行时异常.
	 * @param e the e
	 * @return R
	 */
	@ExceptionHandler(RuntimeException.class)
	@ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
	public R handleGlobalRuntimeException(RuntimeException e) {
		log.error("运行时异常 ex={}", e.getMessage(), e);

		return R.failed(e.getMessage());
	}

	/**
	 * 处理业务校验过程中碰到的非法参数异常 该异常基本由{@link Assert}抛出
	 * @see Assert#hasLength(String, String)
	 * @see Assert#hasText(String, String)
	 * @see Assert#isTrue(boolean, String)
	 * @see Assert#isNull(Object, String)
	 * @see Assert#notNull(Object, String)
	 * @param exception 参数校验异常
	 * @return API返回结果对象包装后的错误输出结果
	 */
	@ExceptionHandler(IllegalArgumentException.class)
	@ResponseStatus(HttpStatus.OK)
	public R handleIllegalArgumentException(IllegalArgumentException exception) {
		log.error("非法参数,ex = {}", exception.getMessage(), exception);
		return R.failed(exception.getMessage());
	}


	/**
	 * AccessDeniedException
	 * @param e the e
	 * @return R
	 */
	@ExceptionHandler(AccessDeniedException.class)
	@ResponseStatus(HttpStatus.FORBIDDEN)
	public R handleAccessDeniedException(AccessDeniedException e) {
		log.warn("无权限异常信息 ex={}", "Access denied");
		return R.failed(e.getLocalizedMessage());
	}

	/**
	 * validation Exception
	 * @param exception e
	 * @return R
	 */
	@ExceptionHandler({ MethodArgumentNotValidException.class })
	@ResponseStatus(HttpStatus.OK)
	public R handleBodyValidException(MethodArgumentNotValidException exception) {
		List<FieldError> fieldErrors = exception.getBindingResult().getFieldErrors();
		log.error("参数绑定异常,ex = {}", fieldErrors.get(0).getDefaultMessage());
		return R.failed(String.format("%s %s", fieldErrors.get(0).getField(), fieldErrors.get(0).getDefaultMessage()));
	}

	/**
	 * validation Exception (以form-data形式传参)
	 * @param exception
	 * @return R
	 */
	@ExceptionHandler({ BindException.class })
	@ResponseStatus(HttpStatus.BAD_REQUEST)
	public R bindExceptionHandler(BindException exception) {
		List<FieldError> fieldErrors = exception.getBindingResult().getFieldErrors();
		log.error("参数绑定异常,ex = {}", fieldErrors.get(0).getDefaultMessage());
		return R.failed(fieldErrors.get(0).getDefaultMessage());
	}

	/**
	 * validationCode Exception
	 * @param exception 异常
	 * @return R
	 */
	@ExceptionHandler({ ValidateCodeException.class })
	@ResponseStatus(HttpStatus.OK)
	public R bindValidateCodeHandler(ValidateCodeException exception) {
		log.warn("验证码异常:{}", exception.getMessage());
		return R.failed(exception.getMessage());
	}

	/**
	 * aes 解密 Exception
	 * @param exception 异常
	 * @return R
	 */
	@ExceptionHandler({ AesDecodeException.class })
	@ResponseStatus(HttpStatus.OK)
	public R<Object> bindAesDecodeException(AesDecodeException exception) {
		log.error("解密异常:{}", exception.getMessage());
		return R.failed(exception.getMessage());
	}


	/**
	 * 自定义异常
	 * @param exception 错误类型
	 * @return {@link R}
	 */
	@ExceptionHandler(GlobalCustomerException.class)
	public R handlerCustomException(GlobalCustomerException exception){
		log.error("自定义异常:{}", exception.getMessage());
		return R.failed(exception.getMessage());
	}

	/**
	 * 参数绑定异常
	 * @param exception 错误类型
	 * @return {@link R}
	 */
	@ExceptionHandler(HttpMessageNotReadableException.class)
	public R handlerParamException(HttpMessageNotReadableException exception){

		log.error("参数绑定异常:{}", exception.getMessage());
		return R.failed(exception.getMessage());
	}

	/**
	 * 远程调用报错
	 * @param exception 错误类型
	 * @return {@link R}
	 */
	@ExceptionHandler(ServletRequestBindingException.class)
	public R handlerParamException(ServletRequestBindingException exception){

		log.error("参数绑定异常:{}", exception.getMessage());
		return R.failed(exception.getMessage());
	}

}
