package com.kxs.common.excel;

import com.alibaba.excel.converters.Converter;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.kxs.common.excel.aop.ResponseExcelReturnValueHandler;
import com.kxs.common.excel.config.ExcelConfigProperties;
import com.kxs.common.excel.config.FileProperties;
import com.kxs.common.excel.enhance.DefaultWriterBuilderEnhancer;
import com.kxs.common.excel.enhance.WriterBuilderEnhancer;
import com.kxs.common.excel.handler.ManySheetWriteHandler;
import com.kxs.common.excel.handler.SheetWriteHandler;
import com.kxs.common.excel.handler.SingleSheetWriteHandler;
import com.kxs.common.excel.head.I18nHeaderCellWriteHandler;
import com.kxs.common.excel.http.RemoteSysService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Bean;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;

import java.util.List;

/**
 * @author 没秃顶的码农
 * @version 1.0
 */
@RequiredArgsConstructor
public class ExcelHandlerConfiguration implements InitializingBean {

	private final ExcelConfigProperties configProperties;

	private final FileProperties ossProperties;
	private OSS ossClient;

	private final ObjectProvider<List<Converter<?>>> converterProvider;

	private final RemoteSysService remoteSysService;

	/**
	 * ExcelBuild增强
	 * @return DefaultWriterBuilderEnhancer 默认什么也不做的增强器
	 */
	@Bean("ExcelWriterBuilderEnhancer")
	@ConditionalOnMissingBean
	public WriterBuilderEnhancer writerBuilderEnhancer() {
		return new DefaultWriterBuilderEnhancer();
	}

	/**
	 * 单sheet 写入处理器
	 */
	@Bean("ExcelSingleSheetWriteHandler")
	@ConditionalOnMissingBean
	public SingleSheetWriteHandler singleSheetWriteHandler() {
		return new SingleSheetWriteHandler(configProperties, ossClient, ossProperties, converterProvider, writerBuilderEnhancer(), remoteSysService);
	}

	/**
	 * 多sheet 写入处理器
	 */
	@Bean("ExcelManySheetWriteHandler")
	@ConditionalOnMissingBean
	public ManySheetWriteHandler manySheetWriteHandler() {
		return new ManySheetWriteHandler(configProperties, ossClient, ossProperties, converterProvider, writerBuilderEnhancer(), remoteSysService);
	}

	/**
	 * 返回Excel文件的 response 处理器
	 * @param sheetWriteHandlerList 页签写入处理器集合
	 * @return ResponseExcelReturnValueHandler
	 */
	@Bean("ExcelResponseExcelReturnValueHandler")
	@ConditionalOnMissingBean
	public ResponseExcelReturnValueHandler responseExcelReturnValueHandler(
			List<SheetWriteHandler> sheetWriteHandlerList) {
		return new ResponseExcelReturnValueHandler(sheetWriteHandlerList, new MappingJackson2HttpMessageConverter());
	}

	/**
	 * excel 头的国际化处理器
	 * @param messageSource 国际化源
	 */
	@Bean("ExcelI18nHeaderCellWriteHandler")
	@ConditionalOnBean(MessageSource.class)
	@ConditionalOnMissingBean
	public I18nHeaderCellWriteHandler i18nHeaderCellWriteHandler(MessageSource messageSource) {
		return new I18nHeaderCellWriteHandler(messageSource);
	}

	@Override
	public void afterPropertiesSet() {
		// 创建OSSClient实例。
		ossClient = new OSSClientBuilder().
				build(ossProperties.getEndpoint(), ossProperties.getAccessKey(), ossProperties.getSecretKey());
	}
}
