package com.kxs.common.pay.service.wechat;

import cn.hutool.core.util.NumberUtil;
import com.github.binarywang.wxpay.bean.notify.SignatureHeader;
import com.github.binarywang.wxpay.bean.notify.WxPayNotifyV3Result;
import com.github.binarywang.wxpay.bean.request.WxPayOrderQueryV3Request;
import com.github.binarywang.wxpay.bean.request.WxPayRefundV3Request;
import com.github.binarywang.wxpay.bean.request.WxPayUnifiedOrderV3Request;
import com.github.binarywang.wxpay.bean.result.WxPayOrderQueryV3Result;
import com.github.binarywang.wxpay.bean.result.WxPayRefundV3Result;
import com.github.binarywang.wxpay.bean.result.WxPayUnifiedOrderV3Result;
import com.github.binarywang.wxpay.bean.result.enums.TradeTypeEnum;
import com.github.binarywang.wxpay.exception.WxPayException;
import com.github.binarywang.wxpay.service.WxPayService;
import com.kxs.common.core.exception.GlobalCustomerException;
import com.kxs.common.pay.enums.PayPlatformEnum;
import com.kxs.common.pay.enums.PayStatusCommEnum;
import com.kxs.common.pay.handlers.PayMessageHandler;
import com.kxs.common.pay.model.*;
import com.kxs.common.pay.service.PayService;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.io.BufferedReader;
import java.io.IOException;
import java.math.BigDecimal;

/**
 * 微信支付 服务
 *
 * @author 没秃顶的码农
 * @date 2024-04-24
 */
@RequiredArgsConstructor
@Slf4j
public class WechatPayService implements PayService {


    private final WxPayService wxPayService;

    @Override
    public PayResponse appPay(PayRequest payReq) {

        try {

            WxPayUnifiedOrderV3Request req = new WxPayUnifiedOrderV3Request();
            wxPayService.createOrderV3(TradeTypeEnum.APP, req);

        } catch (WxPayException e) {
            throw new RuntimeException(e);
        }
        return null;
    }

    @Override
    public PayResponse jsPay(PayRequest payReq) {

        try {

            WxPayUnifiedOrderV3Request req = new WxPayUnifiedOrderV3Request();
            //支付金额
            WxPayUnifiedOrderV3Request.Amount amount = new WxPayUnifiedOrderV3Request.Amount();
            BigDecimal totalAmt = NumberUtil.mul(BigDecimal.valueOf(payReq.getTotalAmount()), 100);
            amount.setTotal(totalAmt.intValue());
            req.setAmount(amount);
            req.setOutTradeNo(payReq.getOutTradeNo());
            req.setDescription(payReq.getSubject());
            //用户信息
            WxPayUnifiedOrderV3Request.Payer payer = new WxPayUnifiedOrderV3Request.Payer();
            payer.setOpenid(payReq.getOpenid());
            req.setPayer(payer);

            req.setAttach(payReq.getPassbackParams());

            WxPayUnifiedOrderV3Result.JsapiResult orderV3 = wxPayService.createOrderV3(TradeTypeEnum.JSAPI, req);

            return PayResponse.builder()
                    .result(orderV3)
                    .payPlatformEnum(PayPlatformEnum.WX).build();

        } catch (WxPayException e) {
            log.error("微信支付异常:{}", payReq.getOutTradeNo(), e);
            throw new GlobalCustomerException("微信支付异常");
        }
    }

    @Override
    public QueryResponse query(QueryRequest queryReq) {

        try {
            WxPayOrderQueryV3Request req = new WxPayOrderQueryV3Request();
            req.setOutTradeNo(queryReq.getOutTradeNo());
            WxPayOrderQueryV3Result response = wxPayService.queryOrderV3(req);
            PayStatusCommEnum payStatusCommEnum;

            switch (response.getTradeState()){
                case "USERPAYING" -> payStatusCommEnum = PayStatusCommEnum.WAIT_BUYER_PAY;
                case "SUCCESS"-> payStatusCommEnum = PayStatusCommEnum.TRADE_SUCCESS;
                default -> payStatusCommEnum = PayStatusCommEnum.TRADE_CLOSED;
            }

            return QueryResponse.builder()
                    .outTradeNo(response.getOutTradeNo())
                    .tradeNo(response.getOutTradeNo())
                    .tradeStatus(payStatusCommEnum)
                    .tradeNo(response.getTransactionId())
                    .payPlatformEnum(PayPlatformEnum.WX).build();
        } catch (WxPayException e) {
            log.error("微信查询订单异常:{}", queryReq.getOutTradeNo(), e);
            throw new GlobalCustomerException(e.getMessage());
        }
    }

    @Override
    public RefundResponse refund(RefundRequest refundReq) {

        try {

            WxPayRefundV3Request req = new WxPayRefundV3Request();
            //支付金额
            WxPayRefundV3Request.Amount amount = new WxPayRefundV3Request.Amount();
            BigDecimal totalAmt = NumberUtil.mul(BigDecimal.valueOf(refundReq.getTotalFee()), 100);
            BigDecimal refundAmt = NumberUtil.mul(BigDecimal.valueOf(refundReq.getRefundFee()), 100);
            amount.setTotal(totalAmt.intValue());
            amount.setRefund(refundAmt.intValue());
            amount.setCurrency("CNY");
            req.setAmount(amount);
            req.setOutTradeNo(refundReq.getOutTradeNo());
            req.setReason(refundReq.getRefundReason());
            req.setOutRefundNo(refundReq.getOutRefundNo());

            WxPayRefundV3Result response = wxPayService.refundV3(req);

            return RefundResponse.builder()
                    .outTradeNo(response.getOutTradeNo())
                    .tradeNo(response.getOutRefundNo())
                    .refundFee(NumberUtil.div(BigDecimal.valueOf(response.getAmount().getRefundFee()), 100))
                    .payPlatformEnum(PayPlatformEnum.WX).build();

        } catch (WxPayException e) {
            log.error("微信退款异常:{}", refundReq.getOutTradeNo(), e);
            throw new GlobalCustomerException(e.getMessage());
        }
    }

    @Override
    public String payBack(HttpServletRequest request) {
        try {

            BufferedReader br = request.getReader();
            String str;
            StringBuilder requestBody = new StringBuilder();
            while((str = br.readLine()) != null){
                requestBody.append(str);
            }
            WxPayNotifyV3Result wxPayNotifyV3Result = wxPayService.parseOrderNotifyV3Result(requestBody.toString(), getRequestHeader(request));
            log.info("开始处理微信回调通知:{}", wxPayNotifyV3Result.getResult().getOutTradeNo());
            WxPayNotifyV3Result.DecryptNotifyResult result = wxPayNotifyV3Result.getResult();
            WxPayMessage wxPayMessage = WxPayMessage.builder()
                    .outTradeNo(result.getOutTradeNo())
                    .tradeState(result.getTradeState())
                    .tradeStateDesc(result.getTradeStateDesc())
                    .successTime(result.getSuccessTime())
                    .bankType(result.getBankType())
                    .attach(result.getAttach())
                    .tradeType(result.getTradeType())
                    .transactionId(result.getTransactionId())
                    .amount(NumberUtil.div(result.getAmount().getTotal(), new BigDecimal(100))).build();
            wxPayMessage.setPayType(PayPlatformEnum.WX);
            PayMessageHandler payMessageHandler = getPayMessageHandler(PayPlatformEnum.WX);

            return payMessageHandler.handle(wxPayMessage, this);

        } catch (WxPayException | IOException e) {
            log.error("微信回调处理异常", e);
            throw new GlobalCustomerException("微信回调异常");
        }

    }
    /**
     * 获取回调请求头：签名相关
     *
     * @param request HttpServletRequest
     * @return SignatureHeader
     */
    private SignatureHeader getRequestHeader(HttpServletRequest request) {
        // 获取通知签名
        String signature = request.getHeader("Wechatpay-Signature");
        String nonce = request.getHeader("Wechatpay-Nonce");
        String serial = request.getHeader("Wechatpay-Serial");
        String timestamp = request.getHeader("Wechatpay-Timestamp");

        SignatureHeader signatureHeader = new SignatureHeader();
        signatureHeader.setSignature(signature);
        signatureHeader.setNonce(nonce);
        signatureHeader.setSerial(serial);
        signatureHeader.setTimeStamp(timestamp);
        return signatureHeader;
    }

}
