package com.kxs.common.security.support.handler;

import cn.hutool.core.text.CharSequenceUtil;
import com.kxs.common.core.constant.CommonConstants;
import com.kxs.common.core.util.R;
import com.kxs.common.core.util.SpringContextHolder;
import com.kxs.common.log.event.SysLogEvent;
import com.kxs.common.log.util.LogTypeEnum;
import com.kxs.common.log.util.SysLogUtils;
import com.kxs.common.security.util.OAuth2ErrorCodesExpand;
import com.kxs.lhb.system.api.model.SysLog;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.http.server.ServletServerHttpResponse;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.oauth2.core.OAuth2AuthenticationException;
import org.springframework.security.oauth2.core.endpoint.OAuth2ParameterNames;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;

import java.io.IOException;

/**
 * @author 没秃顶的码农
 * @date 2022-06-02
 */
@Slf4j
public class SkyAuthenticationFailureEventHandler implements AuthenticationFailureHandler {

    private final MappingJackson2HttpMessageConverter errorHttpResponseConverter = new MappingJackson2HttpMessageConverter();

    /**
     * Called when an authentication attempt fails.
     *
     * @param request   the request during which the authentication attempt occurred.
     * @param response  the response.
     * @param exception the exception which was thrown to reject the authentication
     *                  request.
     */
    @Override
    @SneakyThrows
    public void onAuthenticationFailure(HttpServletRequest request, HttpServletResponse response,
                                        AuthenticationException exception) {
        String username = null;
        if (request.getAttribute(OAuth2ParameterNames.USERNAME) != null) {
            username = request.getAttribute(OAuth2ParameterNames.USERNAME).toString();
        }

        String errorMessage;
        if (exception instanceof OAuth2AuthenticationException authorizationException) {
            errorMessage = CharSequenceUtil.isBlank(authorizationException.getError().getDescription())
                    ? authorizationException.getError().getErrorCode()
                    : authorizationException.getError().getDescription();
        } else {
            errorMessage = exception.getLocalizedMessage();
        }

        log.error("用户：{} 登录失败，异常：{}", username, errorMessage);
        SysLog logVo = SysLogUtils.getSysLog();
        logVo.setTitle("登录失败");
        logVo.setLogType(LogTypeEnum.ERROR.getType());
        logVo.setException(exception.getLocalizedMessage());
        // 发送异步日志事件
        String startTimeStr = request.getHeader(CommonConstants.REQUEST_START_TIME);
        if (CharSequenceUtil.isNotBlank(startTimeStr)) {
            Long startTime = Long.parseLong(startTimeStr);
            Long endTime = System.currentTimeMillis();
            logVo.setTime(endTime - startTime);
        }
        logVo.setCreateBy(username);
        SpringContextHolder.publishEvent(new SysLogEvent(logVo));
        // 写出错误信息
        sendErrorResponse(request, response, exception);
    }

    private void sendErrorResponse(HttpServletRequest request, HttpServletResponse response,
                                   AuthenticationException exception) throws IOException {
        ServletServerHttpResponse httpResponse = new ServletServerHttpResponse(response);
        httpResponse.setStatusCode(HttpStatus.OK);
        String errorMessage;
        R<String> result = new R<>();
        result.setStatus(CommonConstants.FAIL);

        if (exception instanceof OAuth2AuthenticationException authorizationException) {
            errorMessage = CharSequenceUtil.isBlank(authorizationException.getError().getDescription())
                    ? authorizationException.getError().getErrorCode()
                    : authorizationException.getError().getDescription();
            //针对锁定返回423
            if(authorizationException.getError().getErrorCode().equals(OAuth2ErrorCodesExpand.USER_LOCKED)){
//                httpResponse.setStatusCode(HttpStatus.LOCKED);
                result.setStatus(HttpStatus.LOCKED.value());
            }
        } else {
            errorMessage = exception.getLocalizedMessage();
        }
        result.setMsg(errorMessage);
        this.errorHttpResponseConverter.write(result, MediaType.APPLICATION_JSON, httpResponse);
    }

}
