package com.kxs.lhb.basic.api.util;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.io.*;
import java.math.BigInteger;
import java.security.*;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.Arrays;
import java.util.Base64;
import java.util.Enumeration;

/**
 * RSA 实用程序
 *
 * @author Lxq
 * @date 2024-09-02
 */
public class RsaUtil {

    public static final String ALGORITHM = "RSA";
    private static final String SIGNATURE = "SHA256withRSA";
    public static final String CERT_TYPE = "X.509";
    public static final String KEY_TYPE = "PKCS12";


    /**
     * RSA算法规定：待加密的字节数不能超过密钥的长度值除以8再减去11。
     * 而加密后得到密文的字节数，正好是密钥的长度值除以 8。
     */
    private static final int RESERVE_BYTES = 11;
    private static final int BIT = 8;



    public static PublicKey publicKey(String cert) throws NoSuchAlgorithmException, InvalidKeySpecException {
        X509EncodedKeySpec spec = new X509EncodedKeySpec(Base64.getDecoder().decode(cert));
        KeyFactory keyFactory = KeyFactory.getInstance(ALGORITHM);
        return keyFactory.generatePublic(spec);
    }

    public static PublicKey publicKey(File cer) throws FileNotFoundException, CertificateException {
        CertificateFactory certificateFactory = CertificateFactory.getInstance(CERT_TYPE);
        FileInputStream fileInputStream = new FileInputStream(cer);
        X509Certificate x509Certificate = (X509Certificate) certificateFactory.generateCertificate(fileInputStream);
        return x509Certificate.getPublicKey();
    }

    public static PrivateKey privateKey(File pfx, String password) throws KeyStoreException, UnrecoverableKeyException, NoSuchAlgorithmException, CertificateException, IOException {
        char[] p = password.toCharArray();
        KeyStore keyStore = keyStore(pfx, password);
        Enumeration<String> enumeration = keyStore.aliases();
        String alias = null;
        if (enumeration != null && enumeration.hasMoreElements()) {
            alias = enumeration.nextElement();
        }
        return (PrivateKey) keyStore.getKey(alias, p);
    }

    public static PrivateKey privateKey(String cert) throws InvalidKeySpecException, NoSuchAlgorithmException {
        byte[] keyBytes = Base64.getDecoder().decode(cert);
        PKCS8EncodedKeySpec spec = new PKCS8EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(ALGORITHM);
        return keyFactory.generatePrivate(spec);
    }

    private static KeyStore keyStore(File pfx, String password) throws IOException, KeyStoreException, CertificateException, NoSuchAlgorithmException {
        FileInputStream fileInputStream = new FileInputStream(pfx);
        char[] p = password.toCharArray();
        KeyStore keyStore = KeyStore.getInstance(KEY_TYPE);
        keyStore.load(fileInputStream, p);
        return keyStore;
    }

    private static KeyStore keyStore(InputStream inputStream, String password) throws IOException, KeyStoreException, CertificateException, NoSuchAlgorithmException {
        char[] p = password.toCharArray();
        KeyStore keyStore = KeyStore.getInstance(KEY_TYPE);
        keyStore.load(inputStream, p);
        return keyStore;
    }

    public static String encrypt(String cert, String plains) throws NoSuchPaddingException, NoSuchAlgorithmException, InvalidKeyException, IllegalBlockSizeException, BadPaddingException, InvalidKeySpecException {
        RSAPublicKey publicKey = (RSAPublicKey) publicKey(cert);
        BigInteger modulus = publicKey.getModulus();
        int bitLength = modulus.bitLength();
        int block = (bitLength / BIT) - RESERVE_BYTES;
        Cipher cipher = Cipher.getInstance(ALGORITHM);
        cipher.init(Cipher.ENCRYPT_MODE, publicKey);
        byte[] bytes = cipher(cipher, block, plains.getBytes());
        return Base64.getEncoder().encodeToString(bytes);
    }

    public static String encrypt(File cer, String plains) throws NoSuchPaddingException, NoSuchAlgorithmException, InvalidKeyException, IllegalBlockSizeException, BadPaddingException, CertificateException, FileNotFoundException {
        RSAPublicKey publicKey = (RSAPublicKey) publicKey(cer);
        BigInteger modulus = publicKey.getModulus();
        int bitLength = modulus.bitLength();
        int block = (bitLength / BIT) - RESERVE_BYTES;
        Cipher cipher = Cipher.getInstance(ALGORITHM);
        cipher.init(Cipher.ENCRYPT_MODE, publicKey);
        byte[] bytes = cipher(cipher, block, plains.getBytes());
        return Base64.getEncoder().encodeToString(bytes);
    }

    public static String encrypt(PublicKey publicKey, String plains) throws NoSuchPaddingException, NoSuchAlgorithmException, InvalidKeyException, IllegalBlockSizeException, BadPaddingException {
        RSAPublicKey rsaPublicKey = (RSAPublicKey) publicKey;
        BigInteger modulus = rsaPublicKey.getModulus();
        int bitLength = modulus.bitLength();
        int block = (bitLength / BIT) - RESERVE_BYTES;
        Cipher cipher = Cipher.getInstance(ALGORITHM);
        cipher.init(Cipher.ENCRYPT_MODE, rsaPublicKey);
        byte[] bytes = cipher(cipher, block, plains.getBytes());
        return Base64.getEncoder().encodeToString(bytes);
    }

    public static String decrypt(File pfx, String password, String ciphers) throws UnrecoverableKeyException, CertificateException, KeyStoreException, NoSuchAlgorithmException, IOException, NoSuchPaddingException, InvalidKeyException, IllegalBlockSizeException, BadPaddingException {
        RSAPrivateKey privateKey = (RSAPrivateKey) privateKey(pfx, password);
        BigInteger modulus = privateKey.getModulus();
        int bitLength = modulus.bitLength();
        int block = (bitLength / BIT);
        Cipher cipher = Cipher.getInstance(ALGORITHM);
        cipher.init(Cipher.DECRYPT_MODE, privateKey);
        byte[] bytes = cipher(cipher, block, Base64.getDecoder().decode(ciphers));
        return new String(bytes);
    }


    public static String decrypt(PrivateKey privateKey, String ciphers) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, IllegalBlockSizeException, BadPaddingException {
        RSAPrivateKey rsaPrivateKey = (RSAPrivateKey) privateKey;
        BigInteger modulus = rsaPrivateKey.getModulus();
        int bitLength = modulus.bitLength();
        int block = (bitLength / BIT);
        Cipher cipher = Cipher.getInstance(ALGORITHM);
        cipher.init(Cipher.DECRYPT_MODE, rsaPrivateKey);
        byte[] bytes = cipher(cipher, block, Base64.getDecoder().decode(ciphers));
        return new String(bytes);
    }

    public static String decrypt(String cert, String ciphers) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, IllegalBlockSizeException, BadPaddingException, InvalidKeySpecException {
        RSAPrivateKey rsaPrivateKey = (RSAPrivateKey)privateKey(cert);
        BigInteger modulus = rsaPrivateKey.getModulus();
        int bitLength = modulus.bitLength();
        int block = (bitLength / BIT);
        Cipher cipher = Cipher.getInstance(ALGORITHM);
        cipher.init(Cipher.DECRYPT_MODE, rsaPrivateKey);
        byte[] bytes = cipher(cipher, block, Base64.getDecoder().decode(ciphers));
        return new String(bytes);
    }

    public static String sign(File pfx, String password, String data) throws UnrecoverableKeyException, CertificateException, KeyStoreException, NoSuchAlgorithmException, IOException, InvalidKeyException, SignatureException {
        PrivateKey privateKey = privateKey(pfx, password);
        Signature signature = Signature.getInstance(SIGNATURE);
        signature.initSign(privateKey);
        signature.update(data.getBytes());
        return new String(Base64.getEncoder().encode(signature.sign()));
    }

    public static String sign(String cert, String data) throws InvalidKeySpecException, NoSuchAlgorithmException, InvalidKeyException, SignatureException {
        PrivateKey privateKey = privateKey(cert);
        Signature signature = Signature.getInstance(SIGNATURE);
        signature.initSign(privateKey);
        signature.update(data.getBytes());
        return new String(Base64.getEncoder().encode(signature.sign()));
    }

    public static String sign(PrivateKey privateKey, String data) throws NoSuchAlgorithmException, InvalidKeyException, SignatureException {
        RSAPrivateKey rsaPrivateKey = (RSAPrivateKey) privateKey;
        Signature signature = Signature.getInstance(SIGNATURE);
        signature.initSign(rsaPrivateKey);
        signature.update(data.getBytes());
        return new String(Base64.getEncoder().encode(signature.sign()));
    }

    public static boolean verify(String cert, String data, String sign) throws NoSuchAlgorithmException, InvalidKeyException, SignatureException, InvalidKeySpecException {
        PublicKey publicKey = publicKey(cert);
        Signature signature = Signature.getInstance(SIGNATURE);
        signature.initVerify(publicKey);
        signature.update(data.getBytes());
        return signature.verify(Base64.getDecoder().decode(sign.getBytes()));
    }

    public static boolean verify(File cer, String data, String sign) throws CertificateException, NoSuchAlgorithmException, InvalidKeyException, SignatureException, FileNotFoundException {
        PublicKey publicKey = publicKey(cer);
        Signature signature = Signature.getInstance(SIGNATURE);
        signature.initVerify(publicKey);
        signature.update(data.getBytes());
        return signature.verify(Base64.getDecoder().decode(sign.getBytes()));
    }

    public static boolean verify(PublicKey publicKey, String data, String sign) throws NoSuchAlgorithmException, InvalidKeyException, SignatureException {
        RSAPublicKey rsaPublicKey = (RSAPublicKey) publicKey;
        Signature signature = Signature.getInstance(SIGNATURE);
        signature.initVerify(rsaPublicKey);
        signature.update(data.getBytes());
        return signature.verify(Base64.getDecoder().decode(sign.getBytes()));
    }

    /**
     * 分段加密解密
     *
     * @param cipher 加密器
     * @param input  原始数据
     * @return 分段加密或解密后的结果
     * @throws BadPaddingException       e
     * @throws IllegalBlockSizeException e
     */
    private static byte[] cipher(Cipher cipher, int block, byte[] input) throws BadPaddingException, IllegalBlockSizeException {
        int length = input.length;
        int offset = 0;
        byte[] bytes = new byte[0];
        byte[] cache;
        while (length - offset > 0) {
            if (length - offset > block) {
                cache = cipher.doFinal(input, offset, block);
                offset += block;
            } else {
                cache = cipher.doFinal(input, offset, length - offset);
                offset = length;
            }
            bytes = Arrays.copyOf(bytes, bytes.length + cache.length);
            System.arraycopy(cache, 0, bytes, bytes.length - cache.length, cache.length);
        }
        return bytes;
    }

}