package com.kxs.lhb.basic.biz.mq;

import cn.hutool.core.util.StrUtil;
import com.kxs.common.core.constant.CommonConstants;
import com.kxs.common.mq.handlers.IMQSender;
import com.kxs.lhb.basic.api.amqp.RabbitLhbBindQueueMQ;
import com.kxs.lhb.basic.api.constant.BasicConstants;
import com.kxs.lhb.basic.api.model.LhbIsvInfo;
import com.kxs.lhb.basic.api.model.LhbNotify;
import com.kxs.lhb.basic.biz.mq.model.MarkingModel;
import com.kxs.lhb.basic.biz.service.LhbIsvInfoService;
import com.kxs.lhb.basic.biz.service.LhbNotifyService;
import com.kxs.lhb.system.api.util.ParamResolver;
import com.rabbitmq.client.Channel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Objects;

/**
 * rabbit 队列侦听器 绑定推送到第三方队列
 *
 * @author mac
 * @date 2024-09-07
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class RabbiLhbBindQueueListener {

	private final IMQSender mqSender;

	private final LhbIsvInfoService isvInfoService;
	private final LhbNotifyService notifyService;

	/**
	 * 监听 绑定队列
	 *
	 * @param message 消息
	 */
	@RabbitListener(queues = RabbitLhbBindQueueMQ.QUEUE_NAME, ackMode = "MANUAL")
	@RabbitHandler
	public void onMessage(String msg, Message message, Channel channel){
        try {
			RabbitLhbBindQueueMQ.MsgEntity parse = RabbitLhbBindQueueMQ.parse(msg);

			String serverPrivateKey = ParamResolver.getStr("LOCAL_PRIVATE_KEY");

			LhbNotify notify = notifyService.getByNoteId(parse.getNotifyId());
			if(notify == null){
				log.error("推送信息不存在{}", parse.getNotifyId());
				channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
				return;
			}
			if(Objects.equals(notify.getStatus(), CommonConstants.SUCCESS)){
				log.error("推送信息已推送成功{}", parse.getNotifyId());
				channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
				return;
			}
			if(notify.getNotifyCount() > BasicConstants.MAX_NOTIFY_TIMES){
				log.error("推送信息已超过最大推送次数{}", parse.getNotifyId());
				notify.setStatus(-1);
				notifyService.updateById(notify);
				channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
				return;
			}
			LhbIsvInfo isvInfo = isvInfoService.getByIsvNo(notify.getIsvNo());
			if(isvInfo == null){
				log.error("服务商{}信息不存在", notify.getIsvNo());
				channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
				return;
			}
			if(StrUtil.isBlank(isvInfo.getOrderUrl())){
				log.error("服务商{}推送地址为空", notify.getIsvNo());
				return;
			}

			//推送
			boolean result = MarkingModel.builder()
					.content(notify)
					.encrypt(isvInfo.getPublicKey())
					.sign(serverPrivateKey)
					.execute(isvInfo.getBindUrl());
			if (result) {
				log.info("绑定{}推送成功", parse.getNotifyId());
				notify.setStatus(CommonConstants.SUCCESS);
				notify.setNotifyTime(LocalDateTime.now());
			} else {
				notify.setNotifyCount(notify.getNotifyCount() + 1);
				log.error("绑定{}推送失败开始重试第{}次", parse.getNotifyId(), notify.getNotifyCount());
				//时间递增
				mqSender.send(RabbitLhbBindQueueMQ.build(parse), BasicConstants.ROCKET_MQ_DELAY_LEVEL_SECOND[notify.getNotifyCount()]);
			}
			notifyService.updateById(notify);
			channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
        } catch (Exception e) {
			log.error("绑定消费失败{}", msg, e );
			try {
				channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
			} catch (IOException ex) {
				log.error("绑定消费失败" + ex.getMessage(), ex);
			}
        }
    }


}